(* D'aprs le code Lucid Synchrone de Gwenael Delaval *)

(*********************** ENCODEUR GSM AVEC BUFFERS ****************************)
(******************************************************************************)
(* compiler avec les options  -dcstrs -dbuff
   pour voir les contraintes a resoudre et la taille des buffers necessaires 
*)

(* xor *)
let node xor (a, b) = (a && (not b)) || (b && (not a))

(********** split_speech **********)

let node split_speech x = (x_Ia, x_Ib, x_II) where
  rec x_Ia = x when (1^50 0^210)
  and x_Ib = x when (0^50 1^132 0^78)
  and x_II = x when (0^182 1^78) 

(********* cyclic_encoding *********)

(* join data and redondancy *)
let node join_50_3 ( (x(* ::'a on (1^50 0^3) *)) , r0, r1, r2) = x012 where 
  rec x0  = merge (1^50 0) x r0
  and x01 = merge (1^51 0) x0 r1
  and x012 = merge (1^52 0) x01 r2

(* cyclic encoder: x^3 + x + 1 *)
let node cyclic_encoding x = y where 
  rec u0 = false fby (xor(x, u2))  
  and u1 = false fby (xor(u0, u2))
  and u2 = false fby u1
  and y = join_50_3 
      ( x, 
	buffer(u0 when (0^49 1)), 
	buffer(u1 when (0^49 1)),
	buffer(u2 when (0^49 1)) )

(****** convolutional_encoding ******)

(* to merge the two flows of redondancy *)
let node multiplexer (x1, x2) = o where
  rec o = merge (10) x1 x2

(* convolutional encoder: x^4 + x^3 + 1 and x^4 + x^3 + x + 1 *)
let node convolutional_encoding x = y where
  rec x' = merge (1^185 0^4) x false
  and u1 = false fby x'
  and u2 = false fby u1
  and u3 = false fby u2
  and u4 = false fby u3
  and x1 = xor (xor (xor (x', u1), u3), u4)  (* x^4 + x^3 + x + 1 *)
  and x2 = xor (xor (x', u3), u4)            (* x^4 + x^3 + 1 *)
  and y = multiplexer (x1, buffer(x2))

(********** gsm_encoding ***********)

(* sans buffers *)
(* ne fonctionne qu'avec la methode concrete avec unification globale *)
(*
let node encode_without_buffer x = y where
  rec (x_Ia, x_Ib, x_II) = split_speech x
  and y1 = cyclic_encoding x_Ia
  and y1_x_Ib = merge (1^53 0^132) y1 x_Ib
  and y2 = convolutional_encoding y1_x_Ib
  and y = merge (1^378 0^78) y2 x_II
*)

(* avec buffers *)
let node encode_with_3_buffers x = y where
  rec (x_Ia, x_Ib, x_II) = split_speech x
  and y1 = cyclic_encoding (buffer(x_Ia))
  and y1_x_Ib = merge (1^53 0^132) y1 (buffer(x_Ib))
  and y2 = convolutional_encoding y1_x_Ib
  and y = merge (1^378 0^78) y2 (buffer(x_II))


